# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class NEMMosaic(p.MessageType):
    FIELDS = {
        1: ('namespace', p.UnicodeType, 0),
        2: ('mosaic', p.UnicodeType, 0),
        3: ('quantity', p.UVarintType, 0),
    }
