# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class VerifyMessage(p.MessageType):
    FIELDS = {
        1: ('address', p.UnicodeType, 0),
        2: ('signature', p.BytesType, 0),
        3: ('message', p.BytesType, 0),
        4: ('coin_name', p.UnicodeType, 0),  # default='Bitcoin'
    }
    MESSAGE_WIRE_TYPE = 39
