# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p
from .HDNodeType import HDNodeType


class HDNodePathType(p.MessageType):
    FIELDS = {
        1: ('node', HDNodeType, 0),  # required
        2: ('address_n', p.UVarintType, p.FLAG_REPEATED),
    }
