# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class NEMImportanceTransfer(p.MessageType):
    FIELDS = {
        1: ('mode', p.UVarintType, 0),
        2: ('public_key', p.BytesType, 0),
    }
