# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class TxRequestSerializedType(p.MessageType):
    FIELDS = {
        1: ('signature_index', p.UVarintType, 0),
        2: ('signature', p.BytesType, 0),
        3: ('serialized_tx', p.BytesType, 0),
    }
