# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class CosiCommitment(p.MessageType):
    FIELDS = {
        1: ('commitment', p.BytesType, 0),
        2: ('pubkey', p.BytesType, 0),
    }
    MESSAGE_WIRE_TYPE = 72
