# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class RecoveryDevice(p.MessageType):
    FIELDS = {
        1: ('word_count', p.UVarintType, 0),
        2: ('passphrase_protection', p.BoolType, 0),
        3: ('pin_protection', p.BoolType, 0),
        4: ('language', p.UnicodeType, 0),  # default='english'
        5: ('label', p.UnicodeType, 0),
        6: ('enforce_wordlist', p.BoolType, 0),
        8: ('type', p.UVarintType, 0),
        9: ('u2f_counter', p.UVarintType, 0),
        10: ('dry_run', p.BoolType, 0),
    }
    MESSAGE_WIRE_TYPE = 45
