# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class ApplyFlags(p.MessageType):
    FIELDS = {
        1: ('flags', p.UVarintType, 0),
    }
    MESSAGE_WIRE_TYPE = 28
