# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class EthereumTxRequest(p.MessageType):
    FIELDS = {
        1: ('data_length', p.UVarintType, 0),
        2: ('signature_v', p.UVarintType, 0),
        3: ('signature_r', p.BytesType, 0),
        4: ('signature_s', p.BytesType, 0),
    }
    MESSAGE_WIRE_TYPE = 59
