# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class Entropy(p.MessageType):
    FIELDS = {
        1: ('entropy', p.BytesType, 0),  # required
    }
    MESSAGE_WIRE_TYPE = 10
