# Automatically generated by pb2py
Initialize = 0
Ping = 1
Success = 2
Failure = 3
ChangePin = 4
WipeDevice = 5
FirmwareErase = 6
FirmwareUpload = 7
FirmwareRequest = 8
GetEntropy = 9
Entropy = 10
GetPublicKey = 11
PublicKey = 12
LoadDevice = 13
ResetDevice = 14
SignTx = 15
SimpleSignTx = 16
Features = 17
PinMatrixRequest = 18
PinMatrixAck = 19
Cancel = 20
TxRequest = 21
TxAck = 22
CipherKeyValue = 23
ClearSession = 24
ApplySettings = 25
ButtonRequest = 26
ButtonAck = 27
ApplyFlags = 28
GetAddress = 29
Address = 30
SelfTest = 32
BackupDevice = 34
EntropyRequest = 35
EntropyAck = 36
SignMessage = 38
VerifyMessage = 39
MessageSignature = 40
PassphraseRequest = 41
PassphraseAck = 42
PassphraseStateRequest = 77
PassphraseStateAck = 78
EstimateTxSize = 43
TxSize = 44
RecoveryDevice = 45
WordRequest = 46
WordAck = 47
CipheredKeyValue = 48
EncryptMessage = 49
EncryptedMessage = 50
DecryptMessage = 51
DecryptedMessage = 52
SignIdentity = 53
SignedIdentity = 54
GetFeatures = 55
EthereumGetAddress = 56
EthereumAddress = 57
EthereumSignTx = 58
EthereumTxRequest = 59
EthereumTxAck = 60
GetECDHSessionKey = 61
ECDHSessionKey = 62
SetU2FCounter = 63
EthereumSignMessage = 64
EthereumVerifyMessage = 65
EthereumMessageSignature = 66
NEMGetAddress = 67
NEMAddress = 68
NEMSignTx = 69
NEMSignedTx = 70
CosiCommit = 71
CosiCommitment = 72
CosiSign = 73
CosiSignature = 74
NEMDecryptMessage = 75
NEMDecryptedMessage = 76
DebugLinkDecision = 100
DebugLinkGetState = 101
DebugLinkState = 102
DebugLinkStop = 103
DebugLinkLog = 104
DebugLinkMemoryRead = 110
DebugLinkMemory = 111
DebugLinkMemoryWrite = 112
DebugLinkFlashErase = 113
