# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class EstimateTxSize(p.MessageType):
    FIELDS = {
        1: ('outputs_count', p.UVarintType, 0),  # required
        2: ('inputs_count', p.UVarintType, 0),  # required
        3: ('coin_name', p.UnicodeType, 0),  # default='Bitcoin'
    }
    MESSAGE_WIRE_TYPE = 43
