# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p


class HDNodeType(p.MessageType):
    FIELDS = {
        1: ('depth', p.UVarintType, 0),  # required
        2: ('fingerprint', p.UVarintType, 0),  # required
        3: ('child_num', p.UVarintType, 0),  # required
        4: ('chain_code', p.BytesType, 0),  # required
        5: ('private_key', p.BytesType, 0),
        6: ('public_key', p.BytesType, 0),
    }
