# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p
from .MultisigRedeemScriptType import MultisigRedeemScriptType


class TxOutputType(p.MessageType):
    FIELDS = {
        1: ('address', p.UnicodeType, 0),
        2: ('address_n', p.UVarintType, p.FLAG_REPEATED),
        3: ('amount', p.UVarintType, 0),  # required
        4: ('script_type', p.UVarintType, 0),  # required
        5: ('multisig', MultisigRedeemScriptType, 0),
        6: ('op_return_data', p.BytesType, 0),
        7: ('decred_script_version', p.UVarintType, 0),
    }
