# Automatically generated by pb2py
from __future__ import absolute_import
from .. import protobuf as p
from .NEMCosignatoryModification import NEMCosignatoryModification


class NEMAggregateModification(p.MessageType):
    FIELDS = {
        1: ('modifications', NEMCosignatoryModification, p.FLAG_REPEATED),
        2: ('relative_change', p.Sint32Type, 0),
    }
